@@ -908,7 +908,28 @@ clearlooks_draw_optionmenu (cairo_t *cr,
 	params->style_functions->draw_button (cr, colors, params, x, y, width, height);
 	
 	separator.horizontal = FALSE;
-	params->style_functions->draw_separator (cr, &colors->shade[3], params, &separator, x+optionmenu->linepos, y + offset, 2, height - offset*2);
+	params->style_functions->draw_separator (cr, colors, params, &separator, x+optionmenu->linepos, y + offset, 2, height - offset*2);
+}
+
+static void
+clearlooks_draw_menu_item_separator (cairo_t                   *cr,
+                                     const ClearlooksColors    *colors,
+                                     const WidgetParameters    *widget,
+                                     const SeparatorParameters *separator,
+                                     int x, int y, int width, int height)
+{
+	cairo_save (cr);
+	cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
+	ge_cairo_set_color (cr, &colors->shade[5]);
+
+	if (separator->horizontal)
+		cairo_rectangle (cr, x, y, width, 1);
+	else
+		cairo_rectangle (cr, x, y, 1, height);
+
+	cairo_fill      (cr);
+
+	cairo_restore (cr);
 }
 
 static void
@@ -945,7 +966,7 @@ clearlooks_draw_menubar2 (cairo_t *cr,
 	CairoColor lower;
 	cairo_pattern_t *pattern;
 
-	ge_shade_color (&colors->bg[0], 0.95, &lower);
+	ge_shade_color (&colors->bg[0], 0.96, &lower);
 	
 	cairo_translate (cr, x, y);
 	cairo_rectangle (cr, 0, 0, width, height);
@@ -1253,7 +1274,7 @@ clearlooks_draw_tab (cairo_t *cr,
 	
 	if (params->active)
 	{
-		ge_cairo_set_color (cr, border2);	
+		ge_cairo_set_color (cr, border2);
 		cairo_stroke (cr);
 	}
 	else
@@ -1275,13 +1296,14 @@ clearlooks_draw_tab (cairo_t *cr,
 
 static void
 clearlooks_draw_separator (cairo_t *cr,
-                           const CairoColor     *color,
+                           const ClearlooksColors     *colors,
                            const WidgetParameters     *widget,
                            const SeparatorParameters  *separator,
                            int x, int y, int width, int height)
 {
+	CairoColor color = colors->shade[3];
 	CairoColor hilight; 
-	ge_shade_color (color, 1.4, &hilight);
+	ge_shade_color (&color, 1.4, &hilight);
 
 	cairo_save (cr);
 	cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
@@ -1292,14 +1314,14 @@ clearlooks_draw_separator (cairo_t *cr,
 		cairo_translate       (cr, x, y+0.5);
 		
 		cairo_move_to         (cr, 0.0,   0.0);
-		cairo_line_to         (cr, width, 0.0);		
-		ge_cairo_set_color    (cr, color);
+		cairo_line_to         (cr, width, 0.0);
+		ge_cairo_set_color    (cr, &color);
 		cairo_stroke          (cr);
 		
 		cairo_move_to         (cr, 0.0,   1.0);
 		cairo_line_to         (cr, width, 1.0);
 		ge_cairo_set_color    (cr, &hilight);
-		cairo_stroke          (cr);				
+		cairo_stroke          (cr);
 	}
 	else
 	{
@@ -1308,13 +1330,13 @@ clearlooks_draw_separator (cairo_t *cr,
 		
 		cairo_move_to         (cr, 0.0, 0.0);
 		cairo_line_to         (cr, 0.0, height);
-		ge_cairo_set_color  (cr, color);
+		ge_cairo_set_color    (cr, &color);
 		cairo_stroke          (cr);
 		
 		cairo_move_to         (cr, 1.0, 0.0);
 		cairo_line_to         (cr, 1.0, height);
-		ge_cairo_set_color  (cr, &hilight);
-		cairo_stroke          (cr);		
+		ge_cairo_set_color    (cr, &hilight);
+		cairo_stroke          (cr);
 	}
 
 	cairo_restore (cr);
@@ -1376,10 +1398,10 @@ clearlooks_draw_list_view_header (cairo_t *cr,
 		separator.horizontal = FALSE;
 		
 		if (params->ltr)
-			params->style_functions->draw_separator (cr, &colors->shade[3], params, &separator,
+			params->style_functions->draw_separator (cr, colors, params, &separator,
 			                                         width-1.5, 4.0, 2, height-8.0);
 		else
-			params->style_functions->draw_separator (cr, &colors->shade[3], params, &separator,
+			params->style_functions->draw_separator (cr, colors, params, &separator,
 			                                         1.5, 4.0, 2, height-8.0);
 	}
 }
@@ -2215,6 +2237,55 @@ clearlooks_draw_arrow (cairo_t *cr,
 	                        tx, ty, width, height);
 }
 
+static void
+clearlooks_draw_focus (cairo_t *cr,
+                       const ClearlooksColors          *colors,
+                       const WidgetParameters          *widget,
+                       const FocusParameters           *focus,
+                       int x, int y, int width, int height)
+{
+	if (focus->type == CL_FOCUS_COLOR_WHEEL_LIGHT)
+		cairo_set_source_rgb (cr, 0., 0., 0.);
+	else if (focus->type == CL_FOCUS_COLOR_WHEEL_DARK)
+		cairo_set_source_rgb (cr, 1., 1., 1.);
+	else
+		cairo_set_source_rgba (cr,
+		                       colors->fg[widget->state_type].r,
+		                       colors->fg[widget->state_type].g,
+		                       colors->fg[widget->state_type].b,
+		                       0.7);
+
+	cairo_set_line_width (cr, focus->line_width);
+
+	if (focus->dash_list[0])
+	{
+		gint n_dashes = strlen ((gchar *)focus->dash_list);
+		gdouble *dashes = g_new (gdouble, n_dashes);
+		gdouble total_length = 0;
+		gdouble dash_offset;
+		gint i;
+
+		for (i = 0; i < n_dashes; i++)
+		{
+			dashes[i] = focus->dash_list[i];
+			total_length += focus->dash_list[i];
+		}
+
+		dash_offset = -focus->line_width / 2.0;
+		while (dash_offset < 0)
+			dash_offset += total_length;
+
+		cairo_set_dash (cr, dashes, n_dashes, dash_offset);
+		g_free (dashes);
+	}
+
+	cairo_rectangle (cr,
+			 x + focus->line_width / 2.0,
+			 y + focus->line_width / 2.0,
+			 width - focus->line_width, height - focus->line_width);
+	cairo_stroke (cr);
+}
+
 void
 clearlooks_register_style_classic (ClearlooksStyleFunctions *functions)
 {
@@ -2234,6 +2305,7 @@ clearlooks_register_style_classic (ClearlooksStyleFunctions *functions)
 	functions->draw_tab                = clearlooks_draw_tab;
 	functions->draw_frame              = clearlooks_draw_frame;
 	functions->draw_separator          = clearlooks_draw_separator;
+	functions->draw_menu_item_separator = clearlooks_draw_menu_item_separator;
 	functions->draw_list_view_header   = clearlooks_draw_list_view_header;
 	functions->draw_toolbar            = clearlooks_draw_toolbar;
 	functions->draw_menuitem           = clearlooks_draw_menuitem;
@@ -2248,6 +2320,7 @@ clearlooks_register_style_classic (ClearlooksStyleFunctions *functions)
 	functions->draw_handle             = clearlooks_draw_handle;
 	functions->draw_resize_grip        = clearlooks_draw_resize_grip;
 	functions->draw_arrow              = clearlooks_draw_arrow;
+	functions->draw_focus              = clearlooks_draw_focus;
 	functions->draw_checkbox           = clearlooks_draw_checkbox;
 	functions->draw_radiobutton        = clearlooks_draw_radiobutton;
 	functions->draw_shadow             = clearlooks_draw_shadow;
