@@ -48,6 +48,13 @@ typedef enum
 	CL_ORDER_LAST
 } ClearlooksOrder;
 
+typedef enum
+{
+	CL_CONT_NONE       = 0,
+	CL_CONT_LEFT       = 1 << 0,
+	CL_CONT_RIGHT      = 1 << 1
+} ClearlooksContinue;
+
 typedef enum
 {
 	CL_ORIENTATION_LEFT_TO_RIGHT,
@@ -85,6 +92,20 @@ typedef enum
 	CL_ARROW_COMBO
 } ClearlooksArrowType;
 
+typedef enum
+{
+	CL_FOCUS_BUTTON,
+	CL_FOCUS_LABEL,
+	CL_FOCUS_TREEVIEW_HEADER,
+	CL_FOCUS_TREEVIEW_ROW,
+	CL_FOCUS_TREEVIEW_DND,
+	CL_FOCUS_SCALE,
+	CL_FOCUS_TAB,
+	CL_FOCUS_COLOR_WHEEL_DARK,
+	CL_FOCUS_COLOR_WHEEL_LIGHT
+} ClearlooksFocusType;
+
+
 typedef enum
 {
 	CL_DIRECTION_UP,
@@ -157,6 +178,7 @@ typedef struct
 {
 	boolean lower;
 	boolean horizontal;
+	boolean fill_level;
 } SliderParameters;
 
 typedef struct
@@ -188,7 +210,7 @@ typedef struct
 typedef struct
 {
 	CairoCorners    corners;
-	ClearlooksShadowType shadow;	
+	ClearlooksShadowType shadow;
 } ShadowParameters;
 
 typedef struct
@@ -205,7 +227,7 @@ typedef struct
 typedef struct
 {
 	CairoColor         color;
-	ClearlooksJunction junction;       /* On which sides the slider junctions */
+	ClearlooksJunction junction; /* On which sides the slider junctions */
 	boolean            horizontal;
 	boolean            has_color;
 } ScrollBarParameters;
@@ -218,7 +240,7 @@ typedef struct
 
 typedef struct
 {
-	ClearlooksStepper stepper;         /* Which stepper to draw */
+	ClearlooksStepper stepper;  /* Which stepper to draw */
 } ScrollBarStepperParameters;
 
 typedef struct
@@ -244,6 +266,23 @@ typedef struct
 	ClearlooksDirection direction;
 } ArrowParameters;
 
+typedef struct
+{
+	ClearlooksFocusType type;
+	ClearlooksContinue  continue_side;
+	CairoColor          color;
+	boolean             has_color;
+	gint                line_width;
+	guint8*             dash_list;
+	boolean             interior;
+} FocusParameters;
+
+typedef struct
+{
+	int      style;
+	boolean  topmost;
+} ToolbarParameters;
+
 struct _ClearlooksStyleFunctions
 {
 	void (*draw_button)           (cairo_t				*cr,
@@ -294,10 +333,11 @@ struct _ClearlooksStyleFunctions
 	                               const WidgetParameters		*widget,
 	                               const OptionMenuParameters *optionmenu,
 	                               int x, int y, int width, int height);
-					   
-	void (*draw_inset)            (cairo_t *cr, CairoColor bg_color, 
-                                  double x, double y, double w, double h,
-                                  double radius, uint8 corners);
+
+	void (*draw_inset)            (cairo_t				*cr,
+	                                const CairoColor			*bg_color,
+	                                double x, double y, double w, double h,
+	                                double radius, uint8 corners);
 
 	void (*draw_menubar)          (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
@@ -308,7 +348,7 @@ struct _ClearlooksStyleFunctions
 	void (*draw_tab)              (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
-	                               const TabParameters		   *tab,
+	                               const TabParameters		 	*tab,
 	                               int x, int y, int width, int height);
 
 	void (*draw_frame)            (cairo_t				*cr,
@@ -318,7 +358,13 @@ struct _ClearlooksStyleFunctions
 	                               int x, int y, int width, int height);
 
 	void (*draw_separator)        (cairo_t				*cr,
-	                               const CairoColor		      *color,
+	                               const ClearlooksColors		*colors,
+	                               const WidgetParameters		*widget,
+	                               const SeparatorParameters	*separator,
+	                               int x, int y, int width, int height);
+
+	void (*draw_menu_item_separator) (cairo_t			*cr,
+	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
 	                               const SeparatorParameters	*separator,
 	                               int x, int y, int width, int height);
@@ -332,6 +378,7 @@ struct _ClearlooksStyleFunctions
 	void (*draw_toolbar)          (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
+	                               const ToolbarParameters		*toolbar,
 	                               int x, int y, int width, int height);
 
 	void (*draw_menuitem)         (cairo_t				*cr,
@@ -378,6 +425,11 @@ struct _ClearlooksStyleFunctions
 	                               const WidgetParameters		*widget,
 	                               int x, int y, int width, int height);
 
+	void (*draw_tooltip)          (cairo_t				*cr,
+	                               const ClearlooksColors		*colors,
+	                               const WidgetParameters		*widget,
+	                               int x, int y, int width, int height);
+
 	void (*draw_handle)           (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
@@ -390,12 +442,18 @@ struct _ClearlooksStyleFunctions
 	                               const ResizeGripParameters	*grip,
 	                               int x, int y, int width, int height);
 
-	void (*draw_arrow)      (cairo_t				*cr,
-					                       const ClearlooksColors *colors,
-					                       const WidgetParameters *widget,
-					                       const ArrowParameters  *arrow,
-					                       int x, int y, int width, int height);
-	                               
+	void (*draw_arrow)            (cairo_t				*cr,
+	                               const ClearlooksColors		*colors,
+	                               const WidgetParameters		*widget,
+	                               const ArrowParameters		*arrow,
+	                               int x, int y, int width, int height);
+
+	void (*draw_focus)            (cairo_t				*cr,
+	                               const ClearlooksColors		*colors,
+	                               const WidgetParameters		*widget,
+	                               const FocusParameters		*focus,
+	                               int x, int y, int width, int height);
+
 	void (*draw_checkbox)         (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
@@ -411,10 +469,8 @@ struct _ClearlooksStyleFunctions
 	/* Style internal functions */
 	/* XXX: Only used by slider_button, inline it? */
 	void (*draw_shadow)           (cairo_t				*cr,
-                                  const ClearlooksColors		*colors,
-	                               gfloat         radius,
-	                               int				 width,
-	                               int				 height);
+	                               const ClearlooksColors		*colors,
+	                               gfloat radius, int width, int height);
 
 	void (*draw_slider)           (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
@@ -422,9 +478,9 @@ struct _ClearlooksStyleFunctions
 	                               int x, int y, int width, int height);
 
 	void (*draw_gripdots)         (cairo_t *cr,
-                                    const ClearlooksColors *colors, int x, int y,
-                                    int width, int height, int xr, int yr,
-                                    float contrast);
+	                               const ClearlooksColors *colors, int x, int y,
+	                               int width, int height, int xr, int yr,
+	                               float contrast);
 };
 
 
